#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xproto.h>
#include <X11/xpm.h>
#include <X11/extensions/shape.h>

#include "led.xpm"
#include "wmuptime.xpm"
#include "tile.xpm"

#ifdef sun
#include <time.h>
#include <kstat.h>
#endif

XpmAttributes ledatt;
XpmColorSymbol ledcol[4]={{"led_color_back", NULL, 0},
                          {"led_color_high", NULL, 0},
                          {"led_color_med",  NULL, 0},
                          {"led_color_low",  NULL, 0}};

int wmaker = 0;
int ushape = 0;
char ledcolor[7] = "green";

Pixmap upXPM;
Pixmap mask;
Pixmap tile;
Pixmap disp;
Pixmap dmsk;
Pixmap ledXPM;

Atom delwin;
char dpyname[256] = "";
Display *dpy;
Window Win[2],Root;
XWMHints *hints;
GC WinGC, mGC;
int activeWin;
int activeXPM=-1;

char *buf;

int old_secs=-1,old_hours=-1;

void createWin(Window *win){
   XClassHint classHint;
   *win = XCreateSimpleWindow(dpy, Root, 10, 10, 64, 64,0,0,0);
   classHint.res_name = "wmuptime";
   classHint.res_class = "WMUPTIME";
   XSetClassHint(dpy, *win, &classHint);
}


// uptime in sec's
// if ret=0 then err
int uptime()
{
#ifdef linux
   FILE* f;
   int i=0;
   if(!(f=fopen("/proc/uptime","r"))) { fprintf(stderr,"file!err"); return 0;}
   if(!(fgets(buf,256,f))) { fprintf(stderr,"str!err"); return 0;};
   fclose(f);
   while(*(buf+i)!='.') { i++; }
   *(buf+i)='\0';
   return atoi(buf);
#endif
#ifdef sun
  static uint32_t boot_time;
  time_t t;

  if (!boot_time) {
    kstat_ctl_t *kc;
    kstat_t *ksp;
    kstat_named_t *knamed;

    if(!(kc = kstat_open())) { perror("kstat_open"); return 0; }
    if(!(ksp=kstat_lookup(kc, "unix", 0, "system_misc"))) {
      perror("kstat_lookup"); return 0; }
    kstat_read(kc, ksp, NULL);
    if (!(knamed = (kstat_named_t *)kstat_data_lookup(ksp, "boot_time"))) {
      perror("kstat_data_lookup"); return 0; }
    boot_time = knamed->value.ui32;
  }

  time(&t); 
  return(t - boot_time);
#endif
}
unsigned long getColor(char *colorName, float dim)
{
	XColor Color;
	XWindowAttributes Attributes;

	XGetWindowAttributes(dpy, Root, &Attributes);
	Color.pixel = 0;

	XParseColor (dpy, Attributes.colormap, colorName, &Color);
        Color.red=(unsigned short)(Color.red/dim);
        Color.blue=(unsigned short)(Color.blue/dim);
        Color.green=(unsigned short)(Color.green/dim);
        Color.flags=DoRed | DoGreen | DoBlue;
	XAllocColor (dpy, Attributes.colormap, &Color);

	return Color.pixel;
}
void drawText(int x, int y, char *text)
{
   char *chrset="0123456789 dAtnoc-:;";
   char *chrptr;
   int chrindex,i;
   for(i=0;i<strlen(text);i++) 
      if(chrptr=strchr(chrset,text[i]))
         XCopyArea(dpy, ledXPM, disp, WinGC, 8*(chrptr-chrset),0, 7,11, x+8*i,y);  
}

void remakeGFX(XExposeEvent *xeevent)
{
   XCopyArea(dpy,disp,Win[activeWin],WinGC,
	     xeevent->x,
	     xeevent->y,
	     xeevent->width,
	     xeevent->height,
	     xeevent->x,
	     xeevent->y);
}
void update() {
   int upt,days=0,hours=0,mins=0,secs=0;
   
   upt=uptime();
   secs=upt%60;
   mins=(upt/60)%60;
   hours=(upt/3600)%24;
   days=(upt/86400);     // fixed this
   
   if(old_hours!=hours) {
      sprintf(buf,"%03d:%02d",days,hours);
      drawText(8,16,buf); }
   if(old_secs!=secs) {
      sprintf(buf,"%02d:%02d",mins,secs);
      drawText(12,39,buf); }
   if(old_secs!=secs || old_hours!=hours)
     XCopyArea(dpy,disp,Win[activeWin],WinGC,0,0,64,64,0,0);
}

void dotheArgs(int argc, char *argv[]) {
   int i;
   for(i=1;i<argc;i++) {
      if(strcmp(argv[i],"-h")==0 || strcmp(argv[i],"-help")==0 || strcmp(argv[i],"--help")==0){
	 fputs("wmuptime - uptime for WindowMaker\nversion 0.0.2.980909\n",stderr);
	 fputs("by Joakim Elofsson <basemetal@geocities.com>\n",stderr);
	 fputs("http://basemetal.home.ml.org\n\n",stderr);
	 fprintf(stderr,"Usage:\n  %s [options]\n\noptions:\n\n",argv[0]);
	 fputs("  -h | -help | --help    this help screen\n",stderr);
	 fputs("  -w                     WithdrawnState (for wmaker)\n",stderr);
	 fputs("  -s                     shaped window\n",stderr);
	 fputs("  -l led_color           led color (def. green)\n",stderr);
	 fputs("  -display display       select target display\n\n",stderr);
	 exit(0);
      }
      if(strcmp(argv[i],"-w")==0) wmaker=1;
      if(strcmp(argv[i],"-s")==0) ushape=1;
      if(strcmp(argv[i],"-l")==0) {
	 if(i<argc-1) sprintf(ledcolor,"%s",argv[++i]);
	 continue; }
      if(strcmp(argv[i],"-display")==0) {
	 if(i<argc-1) sprintf(dpyname,"%s",argv[++i]);
	 continue; }
   } }

int main(int argc,char *argv[])
{
   XWMHints hints;
   XSizeHints shints;
   unsigned long gcm;
   XGCValues gcv;
   int finished=0,i;
   XEvent event;

   dotheArgs(argc,argv);
   if(!(buf=(char *)malloc(256))) { fprintf(stderr,"mem!err"); exit(1);}
   if(!(dpy=XOpenDisplay(dpyname))) { fprintf(stderr,"no display"); exit(1);}
   delwin = XInternAtom(dpy, "WM_DELETE_WINDOW", False);
   Root=DefaultRootWindow(dpy);
   createWin(&Win[0]);
   createWin(&Win[1]);
   hints.window_group = Win[0];
   if(wmaker) {
      hints.initial_state = WithdrawnState;
      hints.icon_window = Win[1];
      hints.flags = WindowGroupHint | StateHint | IconWindowHint;
      activeWin=1; }
   else {
      hints.initial_state = NormalState;
      hints.flags = WindowGroupHint | StateHint;
      activeWin=0; }
   XSetWMHints(dpy, Win[0], &hints);
   shints.min_width=64;
   shints.min_height=64;
   shints.max_width=64;
   shints.max_height=64;
   shints.flags=PMinSize | PMaxSize;
   XSetWMNormalHints(dpy, Win[0], &shints);
   XSetCommand(dpy, Win[0], argv, argc);
   XStoreName(dpy, Win[0], "wmuptime");
   XSetIconName(dpy, Win[0], "wmuptime");
   gcm = GCForeground|GCBackground|GCGraphicsExposures;
   gcv.graphics_exposures = False;
   WinGC = XCreateGC(dpy, Root, gcm, &gcv);
   XSelectInput(dpy, Win[activeWin], ExposureMask);
   XSetWMProtocols(dpy, Win[activeWin], &delwin, 1);
   XpmCreatePixmapFromData(dpy, Root, wmuptime_xpm, &upXPM, &mask, NULL);
   XpmCreatePixmapFromData(dpy, Root, tile_xpm, &tile, NULL, NULL);
   ledcol[0].pixel=getColor("black",1);
   ledcol[1].pixel=getColor(ledcolor,1);
   ledcol[2].pixel=getColor(ledcolor,1.65);
   ledcol[3].pixel=getColor(ledcolor,2.6);
   ledatt.numsymbols=4;
   ledatt.colorsymbols=ledcol;
   ledatt.valuemask=XpmColorSymbols;
   XpmCreatePixmapFromData(dpy, Root, led_xpm, &ledXPM, NULL, &ledatt);
   disp = XCreatePixmap(dpy, Root, 64, 64, DefaultDepth(dpy,DefaultScreen(dpy)));
   dmsk = XCreatePixmap(dpy, Root, 64, 64, 1);
   mGC = XCreateGC(dpy, dmsk, gcm, &gcv);
   XCopyArea(dpy, mask, dmsk, mGC, 0, 0, 64, 64, 0, 0);
   if(!(wmaker || ushape))
      {
      XCopyArea(dpy, tile, disp, WinGC, 0, 0, 64, 64, 0, 0);
      XSetClipMask(dpy, WinGC, dmsk);
   }
   else XShapeCombineMask(dpy, Win[activeWin], ShapeBounding, 0, 0, dmsk, ShapeSet);
   XCopyArea(dpy, upXPM, disp, WinGC, 0, 0, 64, 64, 0, 0);
   XSetForeground(dpy, WinGC, getColor("black",1));
   XSetForeground(dpy, WinGC, getColor(ledcolor,1));
   XSetClipOrigin(dpy, WinGC, 0, 0);
   XSetClipMask(dpy, WinGC, None);
   update();
   XMapWindow(dpy, Win[0]);
   
   //mainloop
   while(!finished){
      while(XPending(dpy)){
	 XNextEvent(dpy,&event);
	 switch(event.type){
         case Expose:
            remakeGFX((XExposeEvent *)&event);
	    break;
         case ClientMessage:
            if(event.xclient.data.l[0]==delwin) finished=1;
	    break;
	 }
      }
      update();
      usleep(100000);
   }
   XFreeGC(dpy, WinGC);
   XFreeGC(dpy, mGC);
   XFreePixmap(dpy, ledXPM);
   XFreePixmap(dpy, dmsk);
   XFreePixmap(dpy, disp);
   XFreePixmap(dpy, mask);
   XFreePixmap(dpy, upXPM);
   XFreePixmap(dpy, tile);
   XDestroyWindow(dpy, Win[0]);
   XDestroyWindow(dpy, Win[1]);
   XCloseDisplay(dpy);

   return 0;
}



